from typing import List, Literal, Optional
from typing_extensions import Protocol, runtime_checkable

from pydantic import Field
from nonebot.typing import overrides
from nonebot.utils import escape_tag

from nonebot.adapters import Event as BaseEvent

from .message import Message
from .model import (
    Chat,
    Poll,
    User,
    Update,
    PhotoSize,
    PollAnswer,
    InlineQuery,
    CallbackQuery,
    ShippingQuery,
    ChatJoinRequest,
    ForumTopicClosed,
    ForumTopicEdited,
    PreCheckoutQuery,
    ChatMemberUpdated,
    ForumTopicCreated,
    ChosenInlineResult,
    ForumTopicReopened,
    GeneralForumTopicHidden,
    GeneralForumTopicUnhidden,
)


@runtime_checkable
class EventWithChat(Protocol):
    chat: Chat


class Event(BaseEvent):
    telegram_model: Update = Field(default=None)

    @classmethod
    def __parse_event(cls, obj: dict) -> "Event":
        post_type: str = list(obj.keys())[1]
        event_map = {
            "message": MessageEvent,
            "edited_message": EditedMessageEvent,
            "channel_post": MessageEvent,
            "edited_channel_post": EditedMessageEvent,
            "inline_query": InlineQueryEvent,
            "chosen_inline_result": ChosenInlineResultEvent,
            "callback_query": CallbackQueryEvent,
            "shipping_query": ShippingQueryEvent,
            "pre_checkout_query": PreCheckoutQueryEvent,
            "poll": PollEvent,
            "poll_answer": PollAnswerEvent,
            "chat_member": ChatMemberUpdatedEvent,
            "my_chat_member": ChatMemberUpdatedEvent,
            "chat_join_request": ChatJoinRequestEvent,
        }

        event = event_map[post_type].parse_event(obj[post_type])
        setattr(event, "telegram_model", Update.parse_obj(obj))
        return event

    @classmethod
    def _parse_event(cls, obj: dict, failed: set = set()) -> "Event":
        for subclass in cls.__subclasses__():
            if subclass not in failed:
                try:
                    return subclass.parse_event(obj)
                except:
                    pass
        return cls.parse_obj(obj)

    @classmethod
    def parse_event(cls, obj: dict) -> "Event":
        if hasattr(cls, f"_{cls.__name__}__parse_event"):
            return getattr(cls, f"_{cls.__name__}__parse_event")(obj)
        else:
            return cls._parse_event(obj)

    @overrides(BaseEvent)
    def get_type(self) -> str:
        return ""

    @overrides(BaseEvent)
    def get_event_name(self) -> str:
        return ""

    @overrides(BaseEvent)
    def get_event_description(self) -> str:
        return escape_tag(
            str(self.dict(by_alias=True, exclude_none=True, exclude={"telegram_model"}))
        )

    @overrides(BaseEvent)
    def get_user_id(self) -> str:
        raise ValueError("Event has no user!")

    @overrides(BaseEvent)
    def get_session_id(self) -> str:
        raise ValueError("Event has no session!")

    @overrides(BaseEvent)
    def get_message(self) -> Message:
        raise ValueError("Event has no message!")

    @overrides(BaseEvent)
    def is_tome(self) -> bool:
        return False

    def get_message_description(self) -> str:
        msg_string: List[str] = []
        for seg in self.get_message():
            text = escape_tag(repr(seg))
            if seg.type == "text":
                msg_string.append(text)
            else:
                msg_string.append(f"<le>{text}</le>")
        return "".join(msg_string)


class MessageEvent(Event):
    message_id: int
    date: int
    chat: Chat
    forward_from: Optional[User]
    forward_from_chat: Optional[Chat]
    forward_from_message_id: Optional[int]
    forward_signature: Optional[str]
    forward_sender_name: Optional[str]
    forward_date: Optional[int]
    via_bot: Optional[User]
    has_protected_content: Optional[Literal[True]]
    media_group_id: Optional[str]
    author_signature: Optional[str]
    reply_to_message: Optional["MessageEvent"] = None
    message: Message = Message()
    _tome: bool = False

    @classmethod
    def __parse_event(cls, obj: dict) -> "Event":
        message = Message.parse_obj(obj)
        if not message:
            return NoticeEvent.parse_event(obj)
        else:
            reply_to_message = obj.pop("reply_to_message", None)
            message_type = obj["chat"]["type"]
            event_map = {
                "private": PrivateMessageEvent,
                "group": GroupMessageEvent,
                "supergroup": GroupMessageEvent,
                "channel": ChannelPostEvent,
            }
            event = event_map[message_type].parse_event(obj)
            setattr(event, "message", message)
            if reply_to_message:
                setattr(event, "reply_to_message", cls.parse_event(reply_to_message))
            return event

    @overrides(Event)
    def get_type(self) -> str:
        return "message"

    @overrides(Event)
    def get_event_name(self) -> str:
        return "message"

    @overrides(Event)
    def get_message(self) -> Message:
        return self.message

    @overrides(Event)
    def is_tome(self) -> bool:
        return self._tome

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"Message {self.message_id} @[Chat {self.chat.id}]: {self.get_message_description()}"


class PrivateMessageEvent(MessageEvent):
    from_: User = Field(alias="from")

    @overrides(MessageEvent)
    def get_event_name(self) -> str:
        return "message.private"

    @overrides(MessageEvent)
    def get_user_id(self) -> str:
        return str(self.from_.id)

    @overrides(MessageEvent)
    def get_session_id(self) -> str:
        return f"private_{self.chat.id}"

    @overrides(MessageEvent)
    def is_tome(self) -> bool:
        return True

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"Message {self.message_id} from {self.from_.id}: {self.get_message_description()}"


class GroupMessageEvent(MessageEvent):
    from_: User = Field(alias="from")
    sender_chat: Optional[Chat]

    @overrides(MessageEvent)
    def get_event_name(self) -> str:
        return "message.group"

    @overrides(MessageEvent)
    def get_user_id(self) -> str:
        return str(self.from_.id)

    @overrides(MessageEvent)
    def get_session_id(self) -> str:
        return f"group_{self.chat.id}_{self.from_.id}"

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"Message {self.message_id} from {self.from_.id}@[Chat {self.chat.id}]: {self.get_message_description()}"


class ForumTopicMessageEvent(GroupMessageEvent):
    message_thread_id: int

    @overrides(MessageEvent)
    def get_event_name(self) -> str:
        return "message.group.forum_topic"

    @overrides(MessageEvent)
    def get_session_id(self) -> str:
        return f"group_{self.chat.id}_thread{self.message_thread_id}_{self.from_.id}"

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"Message {self.message_id} from {self.from_.id}@[Chat {self.chat.id} Thread {self.message_thread_id}]: {self.get_message_description()}"


class ChannelPostEvent(MessageEvent):
    sender_chat: Optional[Chat]

    @overrides(MessageEvent)
    def get_event_name(self) -> str:
        return "message.channel_post"

    @overrides(MessageEvent)
    def get_session_id(self) -> str:
        return f"channel_{self.chat.id}"


class EditedMessageEvent(Event):
    message_id: int
    date: int
    chat: Chat
    via_bot: Optional[User]
    edit_date: int
    media_group_id: Optional[str]
    author_signature: Optional[str]
    reply_to_message: Optional["MessageEvent"] = None
    message: Message = Message()

    @classmethod
    def __parse_event(cls, obj: dict):
        reply_to_message = obj.pop("reply_to_message", None)
        message_type = obj["chat"]["type"]
        event_map = {
            "private": PrivateEditedMessageEvent,
            "group": GroupEditedMessageEvent,
            "supergroup": GroupEditedMessageEvent,
            "channel": EditedChannelPostEvent,
        }
        event = event_map[message_type].parse_event(obj)
        setattr(event, "message", Message.parse_obj(obj))
        if reply_to_message:
            setattr(
                event, "reply_to_message", MessageEvent.parse_event(reply_to_message)
            )
        return event

    @overrides(Event)
    def get_type(self) -> str:
        return "edit_message"

    @overrides(Event)
    def get_event_name(self) -> str:
        return "edit_message"

    @overrides(Event)
    def get_message(self) -> Message:
        return self.message

    @overrides(Event)
    def get_plaintext(self) -> str:
        return self.message.extract_plain_text()

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"EditedMessage {self.message_id} @[Chat {self.chat.id}]: {self.get_message_description()}"


class PrivateEditedMessageEvent(EditedMessageEvent):
    from_: User = Field(alias="from")
    sender_chat: Optional[Chat]

    @overrides(EditedMessageEvent)
    def get_event_name(self) -> str:
        return "edited_message.private"

    @overrides(EditedMessageEvent)
    def get_user_id(self) -> str:
        return str(self.from_.id)

    @overrides(EditedMessageEvent)
    def get_session_id(self) -> str:
        return f"private_{self.chat.id}"

    @overrides(EditedMessageEvent)
    def is_tome(self) -> bool:
        return True

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"EditedMessage {self.message_id} from {self.from_.id}: {self.get_message_description()}"


class GroupEditedMessageEvent(EditedMessageEvent):
    from_: User = Field(default=None, alias="from")
    sender_chat: Optional[Chat]

    @overrides(EditedMessageEvent)
    def get_event_name(self) -> str:
        return f"edited_message.group"

    @overrides(EditedMessageEvent)
    def get_user_id(self) -> str:
        return str(self.from_.id)

    @overrides(EditedMessageEvent)
    def get_session_id(self) -> str:
        return f"group_{self.chat.id}_{self.from_.id}"

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"EditedMessage {self.message_id} from {self.from_.id}@[Chat {self.chat.id}]: {self.get_message_description()}"


class ForumTopicEditedMessageEvent(GroupEditedMessageEvent):
    message_thread_id: int

    @overrides(MessageEvent)
    def get_event_name(self) -> str:
        return "edited_message.group.forum_topic"

    @overrides(MessageEvent)
    def get_session_id(self) -> str:
        return f"group_{self.chat.id}_thread{self.message_thread_id}_{self.from_.id}"

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"EditedMessage {self.message_id} from {self.from_.id}@[Chat {self.chat.id} Thread {self.message_thread_id}]: {self.get_message_description()}"


class EditedChannelPostEvent(EditedMessageEvent):
    sender_chat: Optional[Chat]

    @overrides(EditedMessageEvent)
    def get_event_name(self) -> str:
        return "edited_message.channel_post"

    @overrides(EditedMessageEvent)
    def get_session_id(self) -> str:
        return f"channel_{self.chat.id}"


class NoticeEvent(Event):
    @classmethod
    def __parse_event(cls, obj: dict) -> "Event":
        if "pinned_message" in obj:
            return PinnedMessageEvent.parse_event(obj)
        elif "new_chat_members" in obj:
            return NewChatMemberEvent.parse_event(obj)
        elif "left_chat_member" in obj:
            return LeftChatMemberEvent.parse_event(obj)
        elif "new_chat_title" in obj:
            return NewChatTitleEvent.parse_event(obj)
        elif "new_chat_photo" in obj:
            return NewChatPhotoEvent.parse_event(obj)
        elif "delete_chat_photo" in obj:
            return DeleteChatPhotoEvent.parse_event(obj)
        elif "forum_topic_created" in obj:
            return ForumTopicCreatedEvent.parse_event(obj)
        elif "forum_topic_edited" in obj:
            return ForumTopicEditedEvent.parse_event(obj)
        elif "forum_topic_closed" in obj:
            return ForumTopicClosedEvent.parse_event(obj)
        elif "forum_topic_reopened" in obj:
            return ForumTopicReopenedEvent.parse_event(obj)
        elif "general_forum_topic_hidden" in obj:
            return GeneralForumTopicHiddenEvent.parse_event(obj)
        elif "general_forum_topic_unhidden" in obj:
            return GeneralForumTopicUnhiddenEvent.parse_event(obj)
        else:
            return cls._parse_event(obj)

    @overrides(Event)
    def get_type(self) -> str:
        return "notice"

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice"


class PinnedMessageEvent(NoticeEvent):
    message_id: int
    from_: User = Field(alias="from")
    sender_chat: Optional[Chat]
    chat: Chat
    date: int
    pinned_message: MessageEvent = Field(default=None)

    @classmethod
    def __parse_event(cls, obj: dict):
        pinned_message = obj.pop("pinned_message")
        event = cls.parse_obj(obj)
        setattr(event, "pinned_message", MessageEvent.parse_event(pinned_message))
        return event

    @overrides(NoticeEvent)
    def get_event_name(self) -> str:
        return "notice.pinned_message"

    @overrides(NoticeEvent)
    def get_message(self) -> Message:
        return self.pinned_message.get_message()

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"PinnedMessage {self.pinned_message.message_id} @[Chat {self.pinned_message.chat.id}]: {self.get_message_description()}"


class NewChatMemberEvent(NoticeEvent):
    message_id: int
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    date: int
    new_chat_members: List[User]

    @overrides(NoticeEvent)
    def get_event_name(self) -> str:
        return "notice.chat_member.new"


class LeftChatMemberEvent(NoticeEvent):
    message_id: int
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    date: int
    left_chat_member: User

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.chat_member.left"


class ChatMemberUpdatedEvent(NoticeEvent, ChatMemberUpdated):
    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.chat_member.updated"


class PollEvent(NoticeEvent, Poll):
    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.poll"


class PollAnswerEvent(NoticeEvent, PollAnswer):
    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.poll_answer"


class NewChatTitleEvent(NoticeEvent):
    date: int
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    new_chat_title: str

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.chat.new_title"


class NewChatPhotoEvent(NoticeEvent):
    date: int
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    new_chat_photo: List[PhotoSize]

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.chat.new_photo"


class DeleteChatPhotoEvent(NoticeEvent):
    date: int
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    delete_chat_photo: Literal[True]

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.chat.delete_photo"


class ForumTopicCreatedEvent(NoticeEvent):
    message_thread_id: int
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    date: int
    forum_topic_created: ForumTopicCreated

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.forum_topic.created"


class ForumTopicEditedEvent(NoticeEvent):
    message_thread_id: int
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    date: int
    forum_topic_edited: ForumTopicEdited

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.forum_topic.edited"


class ForumTopicClosedEvent(NoticeEvent):
    message_thread_id: int
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    date: int
    forum_topic_closed: ForumTopicClosed

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.forum_topic.closed"


class ForumTopicReopenedEvent(NoticeEvent):
    message_thread_id: int
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    date: int
    forum_topic_reopened: ForumTopicReopened

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.forum_topic.reopened"


class GeneralForumTopicHiddenEvent(NoticeEvent):
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    date: int
    general_forum_topic_hidden: GeneralForumTopicHidden

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.general_forum_topic.hidden"


class GeneralForumTopicUnhiddenEvent(NoticeEvent):
    from_: Optional[User] = Field(default=None, alias="from")
    chat: Chat
    date: int
    general_forum_topic_unhidden: GeneralForumTopicUnhidden

    @overrides(Event)
    def get_event_name(self) -> str:
        return "notice.general_forum_topic.unhidden"


class RequestEvent(Event):
    @overrides(Event)
    def get_type(self) -> str:
        return "request"

    @overrides(Event)
    def get_event_name(self) -> str:
        return "request"


class ChatJoinRequestEvent(RequestEvent, ChatJoinRequest):
    @overrides(NoticeEvent)
    def get_event_name(self) -> str:
        return "request.chat_join"


class InlineEvent(Event):
    @overrides(Event)
    def get_type(self) -> str:
        return "inline"

    @overrides(Event)
    def get_event_name(self) -> str:
        return "inline"


class InlineQueryEvent(InlineEvent, InlineQuery):
    @overrides(Event)
    def get_event_name(self) -> str:
        return "inline.query"

    @overrides(Event)
    def get_message(self) -> Message:
        return Message(self.query)

    @overrides(Event)
    def get_plaintext(self) -> str:
        return self.query

    @overrides(Event)
    def is_tome(self) -> bool:
        return True

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"InlineQuery {self.id} from {self.from_.id}: {self.get_message_description()}"


class ChosenInlineResultEvent(InlineEvent, ChosenInlineResult):
    @overrides(Event)
    def get_event_name(self) -> str:
        return "inline.chosen_result"

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"ChosenInlineResult {self.result_id} from {self.from_.id} for Query {self.query}"


class CallbackQueryEvent(InlineEvent, CallbackQuery):
    @overrides(Event)
    def get_event_name(self) -> str:
        return "inline.callback_query"

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"CallbackQuery {self.id} from {self.from_.id}: {self.data}"


# TODO DELAY
class ShippingQueryEvent(Event, ShippingQuery):
    pass


# TODO DELAY
class PreCheckoutQueryEvent(Event, PreCheckoutQuery):
    pass



from typing import List, Optional

from pydantic import Field, BaseModel


class BotConfig(BaseModel):
    """
    Telegram Bot 配置类

    :配置项:
      - ``token``: telegram bot token
      - ``api_server``: 自定义 API 服务器
      - ``is_webhook``: 是否使用 webhook

    """

    token: str
    api_server: str = "https://api.telegram.org/"
    is_webhook: bool = False

    class Config:
        extra = "ignore"
        allow_population_by_field_name = True


class AdapterConfig(BaseModel):
    """
    Telegram Adapter 配置类

    :配置项:

      - ``proxy``: 自定义代理
      - ``telegram_bots`` 机器人单独配置
      - ``telegram_webhook_url``: 自定义 webhook url
    """

    proxy: Optional[str] = Field(default=None, alias="telegram_proxy")
    telegram_bots: List["BotConfig"] = []
    telegram_webhook_url: Optional[str] = None

    class Config:
        extra = "ignore"
        allow_population_by_field_name = True



from pathlib import Path

import pytest
from nonebug import NONEBOT_INIT_KWARGS

import nonebot
import nonebot.adapters

nonebot.adapters.__path__.append(  # type: ignore
    str((Path(__file__).parent.parent / "nonebot" / "adapters").resolve())
)


def pytest_configure(config: pytest.Config) -> None:
    pass


@pytest.fixture(scope="session", autouse=True)
def init_adapter(nonebug_init: None):
    from nonebot.adapters.telegram import Adapter

    driver = nonebot.get_driver()
    driver.register_adapter(Adapter)



import json
from pathlib import Path

import pytest

from nonebot.adapters.telegram.event import Event, PrivateMessageEvent


@pytest.mark.asyncio
async def test_message_parse_obj():
    with (Path(__file__).parent / "updates.json").open("r", encoding="utf8") as f:
        test_updates = json.load(f)

    update_data = test_updates[5]
    event = Event.parse_event(update_data)
    assert isinstance(event, PrivateMessageEvent)
    assert event.message[0].type == "bold"
    assert event.message[1].type == "text"
    assert event.message[2].type == "italic"

    update_data = test_updates[6]
    event = Event.parse_event(update_data)
    assert isinstance(event, PrivateMessageEvent)
    assert event.message[0].type == "audio"

    update_data = test_updates[7]
    event = Event.parse_event(update_data)
    assert isinstance(event, PrivateMessageEvent)
    assert event.message[0].type == "voice"

    update_data = test_updates[8]
    event = Event.parse_event(update_data)
    assert isinstance(event, PrivateMessageEvent)
    assert event.message[0].type == "document"



from typing import List, Union, Literal, Optional

from .model import (
    Chat,
    File,
    Poll,
    User,
    Update,
    Message,
    Sticker,
    InputFile,
    MessageId,
    BotCommand,
    ChatMember,
    ForceReply,
    ForumTopic,
    InputMedia,
    MenuButton,
    StickerSet,
    WebhookInfo,
    InputSticker,
    LabeledPrice,
    MaskPosition,
    GameHighScore,
    MessageEntity,
    BotDescription,
    ChatInviteLink,
    ShippingOption,
    BotCommandScope,
    ChatPermissions,
    InputMediaAudio,
    InputMediaPhoto,
    InputMediaVideo,
    InlineQueryResult,
    SentWebAppMessage,
    UserProfilePhotos,
    InputMediaDocument,
    BotShortDescription,
    ReplyKeyboardMarkup,
    ReplyKeyboardRemove,
    InlineKeyboardMarkup,
    PassportElementError,
    ChatAdministratorRights,
)


class API:
    async def get_updates(
        self,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        timeout: Optional[int] = None,
        allowed_updates: Optional[List[str]] = None,
    ) -> List[Update]:
        ...

    async def set_webhook(
        self,
        url: str,
        certificate: Optional[InputFile] = None,
        ip_address: Optional[str] = None,
        max_connections: Optional[int] = None,
        allowed_updates: Optional[List[str]] = None,
        drop_pending_updates: Optional[bool] = None,
        secret_token: Optional[str] = None,
    ) -> Literal[True]:
        ...

    async def delete_webhook(
        self, drop_pending_updates: Optional[bool] = None
    ) -> Literal[True]:
        ...

    async def get_webhook_info(self) -> WebhookInfo:
        ...

    async def get_me(self) -> User:
        """
        :说明:
          用于测试机器人 Token 的 API
        :返回:
          * ``User``: 机器人本身的 User
        """
        ...

    async def log_out(self) -> Literal[True]:
        ...

    async def close(self) -> Literal[True]:
        ...

    async def send_message(
        self,
        chat_id: Union[int, str],
        text: str,
        message_thread_id: Optional[int] = None,
        parse_mode: Optional[str] = None,
        entities: Optional[List[MessageEntity]] = None,
        disable_web_page_preview: Optional[bool] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def forward_message(
        self,
        chat_id: Union[int, str],
        from_chat_id: Union[int, str],
        message_id: int,
        message_thread_id: Optional[int] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
    ) -> Message:
        ...

    async def copy_message(
        self,
        chat_id: Union[int, str],
        from_chat_id: Union[int, str],
        message_id: int,
        message_thread_id: Optional[int] = None,
        caption: Optional[str] = None,
        parse_mode: Optional[str] = None,
        caption_entities: Optional[List[MessageEntity]] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> MessageId:
        ...

    async def send_photo(
        self,
        chat_id: Union[int, str],
        photo: Union[InputFile, str],
        message_thread_id: Optional[int] = None,
        caption: Optional[str] = None,
        parse_mode: Optional[str] = None,
        caption_entities: Optional[List[MessageEntity]] = None,
        has_spoiler: Optional[bool] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_audio(
        self,
        chat_id: Union[int, str],
        audio: Union[InputFile, str],
        message_thread_id: Optional[int] = None,
        caption: Optional[str] = None,
        parse_mode: Optional[str] = None,
        caption_entities: Optional[List[MessageEntity]] = None,
        duration: Optional[int] = None,
        performer: Optional[str] = None,
        title: Optional[str] = None,
        thumbnail: Optional[Union[InputFile, str]] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_document(
        self,
        chat_id: Union[int, str],
        document: Union[InputFile, str],
        message_thread_id: Optional[int] = None,
        thumbnail: Optional[Union[InputFile, str]] = None,
        caption: Optional[str] = None,
        parse_mode: Optional[str] = None,
        caption_entities: Optional[List[MessageEntity]] = None,
        disable_content_type_detection: Optional[bool] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_video(
        self,
        chat_id: Union[int, str],
        video: Union[InputFile, str],
        message_thread_id: Optional[int] = None,
        duration: Optional[int] = None,
        width: Optional[int] = None,
        height: Optional[int] = None,
        thumbnail: Optional[Union[InputFile, str]] = None,
        caption: Optional[str] = None,
        parse_mode: Optional[str] = None,
        caption_entities: Optional[List[MessageEntity]] = None,
        has_spoiler: Optional[bool] = None,
        supports_streaming: Optional[bool] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_animation(
        self,
        chat_id: Union[int, str],
        animation: Union[InputFile, str],
        message_thread_id: Optional[int] = None,
        duration: Optional[int] = None,
        width: Optional[int] = None,
        height: Optional[int] = None,
        thumbnail: Optional[Union[InputFile, str]] = None,
        caption: Optional[str] = None,
        parse_mode: Optional[str] = None,
        caption_entities: Optional[List[MessageEntity]] = None,
        has_spoiler: Optional[bool] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_voice(
        self,
        chat_id: Union[int, str],
        voice: Union[InputFile, str],
        message_thread_id: Optional[int] = None,
        caption: Optional[str] = None,
        parse_mode: Optional[str] = None,
        caption_entities: Optional[List[MessageEntity]] = None,
        duration: Optional[int] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_video_note(
        self,
        chat_id: Union[int, str],
        video_note: Union[InputFile, str],
        message_thread_id: Optional[int] = None,
        duration: Optional[int] = None,
        length: Optional[int] = None,
        thumbnail: Optional[Union[InputFile, str]] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_media_group(
        self,
        chat_id: Union[int, str],
        media: List[
            Union[InputMediaAudio, InputMediaDocument, InputMediaPhoto, InputMediaVideo]
        ],
        message_thread_id: Optional[int] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
    ) -> List[Message]:
        ...

    async def send_location(
        self,
        chat_id: Union[int, str],
        latitude: float,
        longitude: float,
        message_thread_id: Optional[int] = None,
        horizontal_accuracy: Optional[float] = None,
        live_period: Optional[int] = None,
        heading: Optional[int] = None,
        proximity_alert_radius: Optional[int] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def edit_message_live_location(
        self,
        latitude: float,
        longitude: float,
        chat_id: Optional[Union[int, str]] = None,
        message_id: Optional[int] = None,
        inline_message_id: Optional[str] = None,
        horizontal_accuracy: Optional[float] = None,
        heading: Optional[int] = None,
        proximity_alert_radius: Optional[int] = None,
        reply_markup: Optional[InlineKeyboardMarkup] = None,
    ) -> Union[Message, Literal[True]]:
        ...

    async def stop_message_live_location(
        self,
        chat_id: Optional[Union[int, str]] = None,
        message_id: Optional[int] = None,
        inline_message_id: Optional[str] = None,
        reply_markup: Optional[InlineKeyboardMarkup] = None,
    ) -> Union[Message, Literal[True]]:
        ...

    async def send_venue(
        self,
        chat_id: Union[int, str],
        latitude: float,
        longitude: float,
        title: str,
        address: str,
        message_thread_id: Optional[int] = None,
        foursquare_id: Optional[str] = None,
        foursquare_type: Optional[str] = None,
        google_place_id: Optional[str] = None,
        google_place_type: Optional[str] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_contact(
        self,
        chat_id: Union[int, str],
        phone_number: str,
        first_name: str,
        message_thread_id: Optional[int] = None,
        last_name: Optional[str] = None,
        vcard: Optional[str] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_poll(
        self,
        chat_id: Union[int, str],
        question: str,
        options: List[str],
        message_thread_id: Optional[int] = None,
        is_anonymous: Optional[bool] = None,
        type: Optional[str] = None,
        allows_multiple_answers: Optional[bool] = None,
        correct_option_id: Optional[int] = None,
        explanation: Optional[str] = None,
        explanation_parse_mode: Optional[str] = None,
        explanation_entities: Optional[List[MessageEntity]] = None,
        open_period: Optional[int] = None,
        close_date: Optional[int] = None,
        is_closed: Optional[bool] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_dice(
        self,
        chat_id: Union[int, str],
        message_thread_id: Optional[int] = None,
        emoji: Optional[str] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def send_chat_action(
        self,
        chat_id: Union[int, str],
        action: str,
        message_thread_id: Optional[int] = None,
    ) -> Literal[True]:
        ...

    async def get_user_profile_photos(
        self, user_id: int, offset: Optional[int] = None, limit: Optional[int] = None
    ) -> UserProfilePhotos:
        ...

    async def get_file(self, file_id: str) -> File:
        ...

    async def ban_chat_member(
        self,
        chat_id: Union[int, str],
        user_id: int,
        until_date: Optional[int] = None,
        revoke_messages: Optional[bool] = None,
    ) -> Literal[True]:
        ...

    async def unban_chat_member(
        self,
        chat_id: Union[int, str],
        user_id: int,
        only_if_banned: Optional[bool] = None,
    ) -> Literal[True]:
        ...

    async def restrict_chat_member(
        self,
        chat_id: Union[int, str],
        user_id: int,
        permissions: ChatPermissions,
        use_independent_chat_permissions: Optional[bool] = None,
        until_date: Optional[int] = None,
    ) -> Literal[True]:
        ...

    async def promote_chat_member(
        self,
        chat_id: Union[int, str],
        user_id: int,
        is_anonymous: Optional[bool] = None,
        can_manage_chat: Optional[bool] = None,
        can_post_messages: Optional[bool] = None,
        can_edit_messages: Optional[bool] = None,
        can_delete_messages: Optional[bool] = None,
        can_manage_video_chats: Optional[bool] = None,
        can_restrict_members: Optional[bool] = None,
        can_promote_members: Optional[bool] = None,
        can_change_info: Optional[bool] = None,
        can_invite_users: Optional[bool] = None,
        can_pin_messages: Optional[bool] = None,
        can_manage_topics: Optional[bool] = None,
    ) -> Literal[True]:
        ...

    async def set_chat_administrator_custom_title(
        self, chat_id: Union[int, str], user_id: int, custom_title: str
    ) -> Literal[True]:
        ...

    async def ban_chat_sender_chat(
        self, chat_id: Union[int, str], sender_chat_id: int
    ) -> Literal[True]:
        ...

    async def unban_chat_sender_chat(
        self, chat_id: Union[int, str], sender_chat_id: int
    ) -> Literal[True]:
        ...

    async def set_chat_permissions(
        self,
        chat_id: Union[int, str],
        permissions: ChatPermissions,
        use_independent_chat_permissions: Optional[bool] = None,
    ) -> Literal[True]:
        ...

    async def export_chat_invite_link(self, chat_id: Union[int, str]) -> str:
        ...

    async def create_chat_invite_link(
        self,
        chat_id: Union[int, str],
        name: Optional[str] = None,
        expire_date: Optional[int] = None,
        member_limit: Optional[int] = None,
        creates_join_request: Optional[bool] = None,
    ) -> ChatInviteLink:
        ...

    async def edit_chat_invite_link(
        self,
        chat_id: Union[int, str],
        invite_link: str,
        name: Optional[str] = None,
        expire_date: Optional[int] = None,
        member_limit: Optional[int] = None,
        creates_join_request: Optional[bool] = None,
    ) -> ChatInviteLink:
        ...

    async def revoke_chat_invite_link(
        self, chat_id: Union[int, str], invite_link: str
    ) -> ChatInviteLink:
        ...

    async def approve_chat_join_request(
        self, chat_id: Union[int, str], user_id: int
    ) -> Literal[True]:
        ...

    async def decline_chat_join_request(
        self, chat_id: Union[int, str], user_id: int
    ) -> Literal[True]:
        ...

    async def set_chat_photo(
        self, chat_id: Union[int, str], photo: InputFile
    ) -> Literal[True]:
        ...

    async def delete_chat_photo(self, chat_id: Union[int, str]) -> Literal[True]:
        ...

    async def set_chat_title(
        self, chat_id: Union[int, str], title: str
    ) -> Literal[True]:
        ...

    async def set_chat_description(
        self, chat_id: Union[int, str], description: Optional[str] = None
    ) -> Literal[True]:
        ...

    async def pin_chat_message(
        self,
        chat_id: Union[int, str],
        message_id: int,
        disable_notification: Optional[bool] = None,
    ) -> Literal[True]:
        ...

    async def unpin_chat_message(
        self, chat_id: Union[int, str], message_id: Optional[int] = None
    ) -> Literal[True]:
        ...

    async def unpin_all_chat_messages(self, chat_id: Union[int, str]) -> Literal[True]:
        ...

    async def leave_chat(self, chat_id: Union[int, str]) -> Literal[True]:
        ...

    async def get_chat(self, chat_id: Union[int, str]) -> Chat:
        ...

    async def get_chat_administrators(
        self, chat_id: Union[int, str]
    ) -> List[ChatMember]:
        ...

    async def get_chat_member_count(self, chat_id: Union[int, str]) -> int:
        ...

    async def get_chat_member(
        self, chat_id: Union[int, str], user_id: int
    ) -> ChatMember:
        ...

    async def set_chat_sticker_set(
        self, chat_id: Union[int, str], sticker_set_name: str
    ) -> Literal[True]:
        ...

    async def delete_chat_sticker_set(self, chat_id: Union[int, str]) -> Literal[True]:
        ...

    async def get_forum_topic_icon_stickers(self) -> List[Sticker]:
        ...

    async def create_forum_topic(
        self,
        chat_id: Union[int, str],
        name: str,
        icon_color: Optional[int] = None,
        icon_custom_emoji_id: Optional[str] = None,
    ) -> ForumTopic:
        ...

    async def edit_forum_topic(
        self,
        chat_id: Union[int, str],
        message_thread_id: int,
        name: Optional[str] = None,
        icon_custom_emoji_id: Optional[str] = None,
    ) -> Literal[True]:
        ...

    async def close_forum_topic(
        self, chat_id: Union[int, str], message_thread_id: int
    ) -> Literal[True]:
        ...

    async def reopen_forum_topic(
        self, chat_id: Union[int, str], message_thread_id: int
    ) -> Literal[True]:
        ...

    async def delete_forum_topic(
        self, chat_id: Union[int, str], message_thread_id: int
    ) -> Literal[True]:
        ...

    async def unpin_all_forum_topic_messages(
        self, chat_id: Union[int, str], message_thread_id: int
    ) -> Literal[True]:
        ...

    async def edit_general_forum_topic(
        self, chat_id: Union[int, str], name: str
    ) -> Literal[True]:
        ...

    async def close_general_forum_topic(
        self, chat_id: Union[int, str]
    ) -> Literal[True]:
        ...

    async def reopen_general_forum_topic(
        self, chat_id: Union[int, str]
    ) -> Literal[True]:
        ...

    async def hide_general_forum_topic(self, chat_id: Union[int, str]) -> Literal[True]:
        ...

    async def unhide_general_forum_topic(
        self, chat_id: Union[int, str]
    ) -> Literal[True]:
        ...

    async def answer_callback_query(
        self,
        callback_query_id: str,
        text: Optional[str] = None,
        show_alert: Optional[bool] = None,
        url: Optional[str] = None,
        cache_time: Optional[int] = None,
    ) -> Literal[True]:
        ...

    async def set_my_commands(
        self,
        commands: List[BotCommand],
        scope: Optional[BotCommandScope] = None,
        language_code: Optional[str] = None,
    ) -> Literal[True]:
        ...

    async def delete_my_commands(
        self,
        scope: Optional[BotCommandScope] = None,
        language_code: Optional[str] = None,
    ) -> Literal[True]:
        ...

    async def get_my_commands(
        self,
        scope: Optional[BotCommandScope] = None,
        language_code: Optional[str] = None,
    ) -> List[BotCommand]:
        ...

    async def set_my_description(
        self, description: Optional[str] = None, language_code: Optional[str] = None
    ) -> Literal[True]:
        ...

    async def get_my_description(
        self, language_code: Optional[str] = None
    ) -> BotDescription:
        ...

    async def set_my_short_description(
        self,
        short_description: Optional[str] = None,
        language_code: Optional[str] = None,
    ) -> Literal[True]:
        ...

    async def get_my_short_description(
        self, language_code: Optional[str] = None
    ) -> BotShortDescription:
        ...

    async def set_chat_menu_button(
        self, chat_id: Optional[int] = None, menu_button: Optional[MenuButton] = None
    ) -> Literal[True]:
        ...

    async def get_chat_menu_button(self, chat_id: Optional[int] = None) -> MenuButton:
        ...

    async def set_my_default_administrator_rights(
        self,
        rights: Optional[ChatAdministratorRights] = None,
        for_channels: Optional[bool] = None,
    ) -> Literal[True]:
        ...

    async def get_my_default_administrator_rights(
        self, for_channels: Optional[bool] = None
    ) -> ChatAdministratorRights:
        ...

    async def edit_message_text(
        self,
        text: str,
        chat_id: Optional[Union[int, str]] = None,
        message_id: Optional[int] = None,
        inline_message_id: Optional[str] = None,
        parse_mode: Optional[str] = None,
        entities: Optional[List[MessageEntity]] = None,
        disable_web_page_preview: Optional[bool] = None,
        reply_markup: Optional[InlineKeyboardMarkup] = None,
    ) -> Union[Message, Literal[True]]:
        ...

    async def edit_message_caption(
        self,
        chat_id: Optional[Union[int, str]] = None,
        message_id: Optional[int] = None,
        inline_message_id: Optional[str] = None,
        caption: Optional[str] = None,
        parse_mode: Optional[str] = None,
        caption_entities: Optional[List[MessageEntity]] = None,
        reply_markup: Optional[InlineKeyboardMarkup] = None,
    ) -> Union[Message, Literal[True]]:
        ...

    async def edit_message_media(
        self,
        media: InputMedia,
        chat_id: Optional[Union[int, str]] = None,
        message_id: Optional[int] = None,
        inline_message_id: Optional[str] = None,
        reply_markup: Optional[InlineKeyboardMarkup] = None,
    ) -> Union[Message, Literal[True]]:
        ...

    async def edit_message_reply_markup(
        self,
        chat_id: Optional[Union[int, str]] = None,
        message_id: Optional[int] = None,
        inline_message_id: Optional[str] = None,
        reply_markup: Optional[InlineKeyboardMarkup] = None,
    ) -> Union[Message, Literal[True]]:
        ...

    async def stop_poll(
        self,
        chat_id: Union[int, str],
        message_id: int,
        reply_markup: Optional[InlineKeyboardMarkup] = None,
    ) -> Poll:
        ...

    async def delete_message(
        self, chat_id: Union[int, str], message_id: int
    ) -> Literal[True]:
        ...

    async def send_sticker(
        self,
        chat_id: Union[int, str],
        sticker: Union[InputFile, str],
        message_thread_id: Optional[int] = None,
        emoji: Optional[str] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ] = None,
    ) -> Message:
        ...

    async def get_sticker_set(self, name: str) -> StickerSet:
        ...

    async def get_custom_emoji_stickers(
        self, custom_emoji_ids: List[str]
    ) -> List[Sticker]:
        ...

    async def upload_sticker_file(
        self, user_id: int, sticker: InputFile, sticker_format: str
    ) -> File:
        ...

    async def create_new_sticker_set(
        self,
        user_id: int,
        name: str,
        title: str,
        stickers: List[InputSticker],
        sticker_format: str,
        sticker_type: Optional[str] = None,
        needs_repainting: Optional[bool] = None,
    ) -> Literal[True]:
        ...

    async def add_sticker_to_set(
        self,
        user_id: int,
        name: str,
        sticker: InputSticker,
    ) -> Literal[True]:
        ...

    async def set_sticker_position_in_set(
        self, sticker: str, position: int
    ) -> Literal[True]:
        ...

    async def delete_sticker_from_set(self, sticker: str) -> Literal[True]:
        ...

    async def set_sticker_emoji_list(
        self, sticker: str, emoji_list: List[str]
    ) -> Literal[True]:
        ...

    async def set_sticker_keywords(
        self, sticker: str, keywords: Optional[List[str]] = None
    ) -> Literal[True]:
        ...

    async def set_sticker_mask_position(
        self, sticker: str, mask_position: Optional[MaskPosition] = None
    ) -> Literal[True]:
        ...

    async def set_sticker_set_title(self, name: str, title: str) -> Literal[True]:
        ...

    async def set_sticker_set_thumbnail(
        self, name: str, user_id: int, thumbnail: Optional[Union[InputFile, str]] = None
    ) -> Literal[True]:
        ...

    async def set_custom_emoji_sticker_set_thumbnail(
        self, name: str, custom_emoji_id: Optional[str] = None
    ) -> Literal[True]:
        ...

    async def delete_sticker_set(self, name: str) -> Literal[True]:
        ...

    async def answer_inline_query(
        self,
        inline_query_id: str,
        results: List[InlineQueryResult],
        cache_time: Optional[int] = None,
        is_personal: Optional[bool] = None,
        next_offset: Optional[str] = None,
        switch_pm_text: Optional[str] = None,
        switch_pm_parameter: Optional[str] = None,
    ) -> Literal[True]:
        ...

    async def answer_web_app_query(
        self, web_app_query_id: str, result: InlineQueryResult
    ) -> SentWebAppMessage:
        ...

    async def send_invoice(
        self,
        chat_id: Union[int, str],
        title: str,
        description: str,
        payload: str,
        provider_token: str,
        currency: str,
        prices: List[LabeledPrice],
        message_thread_id: Optional[int] = None,
        max_tip_amount: Optional[int] = None,
        suggested_tip_amounts: Optional[List[int]] = None,
        start_parameter: Optional[str] = None,
        provider_data: Optional[str] = None,
        photo_url: Optional[str] = None,
        photo_size: Optional[int] = None,
        photo_width: Optional[int] = None,
        photo_height: Optional[int] = None,
        need_name: Optional[bool] = None,
        need_phone_number: Optional[bool] = None,
        need_email: Optional[bool] = None,
        need_shipping_address: Optional[bool] = None,
        send_phone_number_to_provider: Optional[bool] = None,
        send_email_to_provider: Optional[bool] = None,
        is_flexible: Optional[bool] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[InlineKeyboardMarkup] = None,
    ) -> Message:
        ...

    async def create_invoice_link(
        self,
        title: str,
        description: str,
        payload: str,
        provider_token: str,
        currency: str,
        prices: List[LabeledPrice],
        max_tip_amount: Optional[int] = None,
        suggested_tip_amounts: Optional[List[int]] = None,
        provider_data: Optional[str] = None,
        photo_url: Optional[str] = None,
        photo_size: Optional[int] = None,
        photo_width: Optional[int] = None,
        photo_height: Optional[int] = None,
        need_name: Optional[bool] = None,
        need_phone_number: Optional[bool] = None,
        need_email: Optional[bool] = None,
        need_shipping_address: Optional[bool] = None,
        send_phone_number_to_provider: Optional[bool] = None,
        send_email_to_provider: Optional[bool] = None,
        is_flexible: Optional[bool] = None,
    ) -> str:
        ...

    async def answer_shipping_query(
        self,
        shipping_query_id: str,
        ok: bool,
        shipping_options: Optional[List[ShippingOption]] = None,
        error_message: Optional[str] = None,
    ) -> Literal[True]:
        ...

    async def answer_pre_checkout_query(
        self, pre_checkout_query_id: str, ok: bool, error_message: Optional[str] = None
    ) -> Literal[True]:
        ...

    async def set_passport_data_errors(
        self, user_id: int, errors: List[PassportElementError]
    ) -> Literal[True]:
        ...

    async def send_game(
        self,
        chat_id: int,
        game_short_name: str,
        message_thread_id: Optional[int] = None,
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        reply_to_message_id: Optional[int] = None,
        allow_sending_without_reply: Optional[bool] = None,
        reply_markup: Optional[InlineKeyboardMarkup] = None,
    ) -> Message:
        ...

    async def set_game_score(
        self,
        user_id: int,
        score: int,
        force: Optional[bool] = None,
        disable_edit_message: Optional[bool] = None,
        chat_id: Optional[int] = None,
        message_id: Optional[int] = None,
        inline_message_id: Optional[str] = None,
    ) -> Union[Message, Literal[True]]:
        ...

    async def get_game_high_scores(
        self,
        user_id: int,
        chat_id: Optional[int] = None,
        message_id: Optional[int] = None,
        inline_message_id: Optional[str] = None,
    ) -> List[GameHighScore]:
        ...
